@@ -48,6 +48,10 @@ leave1out.rma.peto <- function(x, digits, transf, targs, progbar=FALSE, ...) {
    I2   <- rep(NA_real_, x$k.f)
    H2   <- rep(NA_real_, x$k.f)
 
+   ### elements that need to be returned
+
+   outlist <- "beta=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, QE=QE, QEp=QEp, tau2=tau2, I2=I2, H2=H2"
+
    ### note: skipping NA cases
 
    if (progbar)
@@ -61,7 +65,8 @@ leave1out.rma.peto <- function(x, digits, transf, targs, progbar=FALSE, ...) {
       if (!x$not.na[i])
          next
 
-      res <- try(suppressWarnings(rma.peto(ai=x$ai.f, bi=x$bi.f, ci=x$ci.f, di=x$di.f, add=x$add, to=x$to, drop00=x$drop00, level=x$level, subset=-i)), silent=TRUE)
+      args <- list(ai=x$ai.f, bi=x$bi.f, ci=x$ci.f, di=x$di.f, add=x$add, to=x$to, drop00=x$drop00, level=x$level, subset=-i, outlist=outlist)
+      res <- try(suppressWarnings(.do.call(rma.peto, args)), silent=TRUE)
 
       if (inherits(res, "try-error"))
          next
