@@ -10,9 +10,20 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
       digits <- .get.digits(digits=digits, xdigits=x$digits, dmiss=FALSE)
    }
 
+   footsym <- .get.footsym()
+
+   ddd <- list(...)
+
+   .chkdots(ddd, c("num", "legend"))
+
+   if (is.null(ddd$legend)) {
+      legend <- ifelse(inherits(x, "robust.rma"), TRUE, FALSE)
+   } else {
+      legend <- .isTRUE(ddd$legend)
+   }
+
    if (inherits(x, "rma.uni.trimfill")) {
-      if (!exists(".rmspace"))
-         cat("\n")
+      .space()
       cat(mstyle$text(paste0("Estimated number of missing studies on the ", x$side, " side: ")))
       cat(mstyle$result(paste0(x$k0, " (SE = ", ifelse(is.na(x$se.k0), NA, .fcf(x$se.k0, digits[["se"]])), ")")))
       cat("\n")
@@ -22,12 +33,10 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
          cat(mstyle$result(paste0("p-val ", .pval(x$p.k0, digits[["pval"]], showeq=TRUE, sep=" "))))
          cat("\n")
       }
-      if (exists(".rmspace"))
-         cat("\n")
+      .space(FALSE)
    }
 
-   if (!exists(".rmspace"))
-      cat("\n")
+   .space()
 
    if (x$model == "rma.ls") {
 
@@ -75,6 +84,7 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
       names(fs) <- c("logLik", "deviance", "AIC", "BIC", "AICc")
       cat("\n")
       tmp <- capture.output(print(fs, quote=FALSE, print.gap=2))
+      tmp[1] <- paste0(tmp[1], "\u200b")
       .print.table(tmp, mstyle)
    }
 
@@ -124,13 +134,13 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
          }
       }
 
-      if (!is.element(x$method, c("FE","EE","CE")) && !x$int.only && !is.null(x$R2)) {
+      if (!x$int.only && !is.null(x$R2)) {
          cat(mstyle$text("R^2 (amount of heterogeneity accounted for):            "))
          cat(mstyle$result(paste0(ifelse(is.na(x$R2), NA, .fcf(x$R2, 2)), "%")))
          cat("\n")
       }
 
-      if (!is.element(x$method, c("FE","EE","CE")) || !is.na(x$I2) || !is.na(x$H2) || (!is.element(x$method, c("FE","EE","CE")) && !x$int.only && !is.null(x$R2)))
+      if (!is.element(x$method, c("FE","EE","CE")) || !is.na(x$I2) || !is.na(x$H2) || !is.null(x$R2))
          cat("\n")
 
    }
@@ -161,14 +171,34 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
       cat("\n\n")
    }
 
+   if (inherits(x, "robust.rma")) {
+
+      cat(mstyle$text("Number of estimates:   "))
+      cat(mstyle$result(x$k))
+      cat("\n")
+      cat(mstyle$text("Number of clusters:    "))
+      cat(mstyle$result(x$n))
+      cat("\n")
+
+      cat(mstyle$text("Estimates per cluster: "))
+      if (all(x$tcl[1] == x$tcl)) {
+         cat(mstyle$result(x$tcl[1]))
+      } else {
+         cat(mstyle$result(paste0(min(x$tcl), "-", max(x$tcl), " (mean: ", .fcf(mean(x$tcl), digits=2), ", median: ", round(median(x$tcl), digits=2), ")")))
+      }
+      cat("\n\n")
+
+   }
+
    if (x$p > 1L && !is.na(x$QM)) {
       if (x$model == "rma.ls") {
          cat(mstyle$section(paste0("Test of Location Coefficients (coefficient", ifelse(x$m == 1, " ", "s "), .format.btt(x$btt),"):")))
       } else {
-         cat(mstyle$section(paste0("Test of Moderators (coefficient", ifelse(x$m == 1, " ", "s "), .format.btt(x$btt),"):")))      }
+         cat(mstyle$section(paste0("Test of Moderators (coefficient", ifelse(x$m == 1, " ", "s "), .format.btt(x$btt),"):", ifelse(inherits(x, "robust.rma"), footsym[1], ""))))
+      }
       cat("\n")
       if (is.element(x$test, c("knha","adhoc","t"))) {
-         cat(mstyle$result(paste0("F(df1 = ", x$QMdf[1], ", df2 = ", x$QMdf[2], ") = ", .fcf(x$QM, digits[["test"]]), ", p-val ", .pval(x$QMp, digits[["pval"]], showeq=TRUE, sep=" "))))
+         cat(mstyle$result(paste0("F(df1 = ", x$QMdf[1], ", df2 = ", round(x$QMdf[2], 2), ") = ", .fcf(x$QM, digits[["test"]]), ", p-val ", .pval(x$QMp, digits[["pval"]], showeq=TRUE, sep=" "))))
       } else {
          cat(mstyle$result(paste0("QM(df = ", x$QMdf[1], ") = ", .fcf(x$QM, digits[["test"]]), ", p-val ", .pval(x$QMp, digits[["pval"]], showeq=TRUE, sep=" "))))
       }
@@ -177,6 +207,8 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
 
    if (is.element(x$test, c("knha","adhoc","t"))) {
       res.table <- data.frame(estimate=.fcf(c(x$beta), digits[["est"]]), se=.fcf(x$se, digits[["se"]]), tval=.fcf(x$zval, digits[["test"]]), df=round(x$ddf,2), pval=.pval(x$pval, digits[["pval"]]), ci.lb=.fcf(x$ci.lb, digits[["ci"]]), ci.ub=.fcf(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
+      if (inherits(x, "robust.rma"))
+         colnames(res.table)[c(2:7)] <- paste0(colnames(res.table)[c(2:7)], footsym[1])
    } else {
       res.table <- data.frame(estimate=.fcf(c(x$beta), digits[["est"]]), se=.fcf(x$se, digits[["se"]]), zval=.fcf(x$zval, digits[["test"]]), pval=.pval(x$pval, digits[["pval"]]), ci.lb=.fcf(x$ci.lb, digits[["ci"]]), ci.ub=.fcf(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
    }
@@ -187,12 +219,8 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
       colnames(res.table)[ncol(res.table)] <- ""
    }
 
-   ddd <- list(...)
-
-   .chkdots(ddd, c("num"))
-
    if (.isTRUE(ddd$num))
-      rownames(res.table) <- paste0(1:nrow(res.table), ") ", rownames(res.table))
+      rownames(res.table) <- paste0(seq_len(nrow(res.table)), ") ", rownames(res.table))
 
    if (x$int.only)
       res.table <- res.table[1,]
@@ -208,6 +236,7 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
    } else {
       tmp <- capture.output(print(res.table, quote=FALSE, right=TRUE, print.gap=2))
    }
+   tmp[1] <- paste0(tmp[1], "\u200b")
    .print.table(tmp, mstyle)
 
    if (x$model == "rma.ls") {
@@ -217,14 +246,14 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
          cat(mstyle$section(paste0("Test of Scale Coefficients (coefficient", ifelse(x$m.alpha == 1, " ", "s "), .format.btt(x$att),"):")))
          cat("\n")
          if (is.element(x$test, c("knha","adhoc","t"))) {
-            cat(mstyle$result(paste0("F(df1 = ", x$QSdf[1], ", df2 = ", x$QSdf[2], ") = ", .fcf(x$QS, digits[["test"]]), ", p-val ", .pval(x$QSp, digits[["pval"]], showeq=TRUE, sep=" "))))
+            cat(mstyle$result(paste0("F(df1 = ", x$QSdf[1], ", df2 = ", round(x$QSdf[2], 2), ") = ", .fcf(x$QS, digits[["test"]]), ", p-val ", .pval(x$QSp, digits[["pval"]], showeq=TRUE, sep=" "))))
          } else {
             cat(mstyle$result(paste0("QS(df = ", x$QSdf[1], ") = ", .fcf(x$QS, digits[["test"]]), ", p-val ", .pval(x$QSp, digits[["pval"]], showeq=TRUE, sep=" "))))
          }
          cat("\n")
       }
 
-      if (x$test == "t") {
+      if (is.element(x$test, c("knha","adhoc","t"))) {
          res.table <- data.frame(estimate=.fcf(c(x$alpha), digits[["est"]]), se=.fcf(x$se.alpha, digits[["se"]]), tval=.fcf(x$zval.alpha, digits[["test"]]), df=round(x$ddf.alpha, 2), pval=.pval(x$pval.alpha, digits[["pval"]]), ci.lb=.fcf(x$ci.lb.alpha, digits[["ci"]]), ci.ub=.fcf(x$ci.ub.alpha, digits[["ci"]]), stringsAsFactors=FALSE)
       } else {
          res.table <- data.frame(estimate=.fcf(c(x$alpha), digits[["est"]]), se=.fcf(x$se.alpha, digits[["se"]]), zval=.fcf(x$zval.alpha, digits[["test"]]), pval=.pval(x$pval.alpha, digits[["pval"]]), ci.lb=.fcf(x$ci.lb.alpha, digits[["ci"]]), ci.ub=.fcf(x$ci.ub.alpha, digits[["ci"]]), stringsAsFactors=FALSE)
@@ -236,13 +265,13 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
          colnames(res.table)[ncol(res.table)] <- ""
       }
 
-      for (j in 1:nrow(res.table)) {
+      for (j in seq_len(nrow(res.table))) {
          res.table[j, is.na(res.table[j,])]  <- ifelse(x$alpha.fix[j], "---", "NA")
          res.table[j, res.table[j,] == "NA"] <- ifelse(x$alpha.fix[j], "---", "NA")
       }
 
       if (.isTRUE(ddd$num))
-         rownames(res.table) <- paste0(1:nrow(res.table), ") ", rownames(res.table))
+         rownames(res.table) <- paste0(seq_len(nrow(res.table)), ") ", rownames(res.table))
 
       if (length(x$alpha) == 1L)
          res.table <- res.table[1,]
@@ -256,6 +285,7 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
       } else {
          tmp <- capture.output(print(res.table, quote=FALSE, right=TRUE, print.gap=2))
       }
+      tmp[1] <- paste0(tmp[1], "\u200b")
       .print.table(tmp, mstyle)
 
    }
@@ -286,7 +316,7 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
          colnames(res.table)[ncol(res.table)] <- ""
       }
 
-      for (j in 1:nrow(res.table)) {
+      for (j in seq_len(nrow(res.table))) {
          res.table[j, is.na(res.table[j,])]  <- ifelse(x$delta.fix[j], "---", "NA")
          res.table[j, res.table[j,] == "NA"] <- ifelse(x$delta.fix[j], "---", "NA")
       }
@@ -303,18 +333,42 @@ print.rma.uni <- function(x, digits, showfit=FALSE, signif.stars=getOption("show
       } else {
          tmp <- capture.output(print(res.table, quote=FALSE, right=TRUE, print.gap=2))
       }
+      tmp[1] <- paste0(tmp[1], "\u200b")
       .print.table(tmp, mstyle)
 
    }
 
+   if (signif.legend || legend) {
+      cat("\n")
+      cat(mstyle$legend("---"))
+   }
+
    if (signif.legend) {
       cat("\n")
-      cat(mstyle$legend("---\nSignif. codes: "), mstyle$legend(attr(signif, "legend")))
+      cat(mstyle$legend("Signif. codes: "), mstyle$legend(attr(signif, "legend")))
       cat("\n")
    }
 
-   if (!exists(".rmspace"))
+   if (inherits(x, "robust.rma") && legend) {
       cat("\n")
+      cat(mstyle$legend(paste0(footsym[2], " results based on cluster-robust inference (var-cov estimator: ", x$vbest)))
+      if (x$robumethod == "default") {
+         cat(mstyle$legend(","))
+         cat("\n")
+         cat(mstyle$legend(paste0("   approx. ", ifelse(x$int.only, "t-test and confidence interval", "t/F-tests and confidence intervals"), ", dfs = residual method)")))
+      } else {
+         if (x$coef_test == "Satterthwaite" && x$conf_test == "Satterthwaite" && x$wald_test == "HTZ") {
+            cat(mstyle$legend(","))
+            cat("\n")
+            cat(mstyle$legend(paste0("   approx. ", ifelse(x$int.only, "t-test and confidence interval", "t/F-tests and confidence intervals"), ", dfs = Satterthwaite method)")))
+         } else {
+            cat(mstyle$legend(")"))
+         }
+      }
+      cat("\n")
+   }
+
+   .space()
 
    invisible()
 
