@@ -5,7 +5,7 @@ rstandard.rma.mv <- function(model, digits, cluster, ...) {
    .chkclass(class(model), must="rma.mv", notav="robust.rma")
 
    na.act <- getOption("na.action")
-   on.exit(options(na.action=na.act))
+   on.exit(options(na.action=na.act), add=TRUE)
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
       stop(mstyle$stop("Unknown 'na.action' specified under options()."))
@@ -20,8 +20,12 @@ rstandard.rma.mv <- function(model, digits, cluster, ...) {
 
    misscluster <- ifelse(missing(cluster), TRUE, FALSE)
 
-   if (misscluster)
+   if (misscluster) {
       cluster <- seq_len(x$k.all)
+   } else {
+      mf <- match.call()
+      cluster <- .getx("cluster", mf=mf, data=x$data)
+   }
 
    #########################################################################
 
@@ -32,6 +36,9 @@ rstandard.rma.mv <- function(model, digits, cluster, ...) {
    # the same subsetting and removing of missings (if necessary) as was done
    # during model fitting
 
+   if (length(cluster) != x$k.all)
+      stop(mstyle$stop(paste0("Length of variable specified via 'cluster' (", length(cluster), ") does not match length of data (", x$k.all, ").")))
+
    if (!is.null(x$subset))
       cluster <- cluster[x$subset]
 
@@ -47,9 +54,6 @@ rstandard.rma.mv <- function(model, digits, cluster, ...) {
    if (length(cluster.f) == 0L)
       stop(mstyle$stop(paste0("Cannot find 'cluster' variable (or it has zero length).")))
 
-   if (length(cluster) != x$k)
-      stop(mstyle$stop(paste0("Length of variable specified via 'cluster' (", length(cluster), ") does not match length of data (", x$k, ").")))
-
    #########################################################################
 
    options(na.action="na.omit")
